@@ -1,5 +1,5 @@
 /*
- * Copyright 1996-2002 Hans Reiser
+ * Copyright 1996-2003 Hans Reiser
  */
 
 #define _GNU_SOURCE
@@ -30,14 +30,14 @@ int main (int argc, char * argv []);
 /*
  * modes
  */
-#define DO_NOTHING              0 /* -a specified */
+#define DO_NOTHING              0
 #define FSCK_CHECK              1
 #define FSCK_FIX_FIXABLE        2
 #define FSCK_SB                 3
 #define FSCK_REBUILD            4
 #define FSCK_ROLLBACK_CHANGES   5
 #define FSCK_CLEAN_ATTRIBUTES	7
-#define AUTO                    8 /* -p specified */
+#define FSCK_AUTO               8 /* -a || -p specified */
 
 /* temporary */
 #define DO_TEST                 9
@@ -117,11 +117,11 @@ void rewrite_object (struct item_head * ih, int do_remap);
 void pass_2_take_bad_blocks_put_into_tree (void);
 /*int is_remapped (struct item_head * ih);*/
 void link_relocated_files (void);
-//void relocate_file (struct item_head * ih, int change_ih);
 int should_relocate (struct item_head * ih);
 void relocate_dir (struct item_head * ih, int change_ih);
-__u32 objectid_for_relocation (struct key * key);
-int should_be_relocated (struct key * key);
+
+extern __u32 objectid_for_relocation (struct key * key);
+extern void linked_already(struct key *new_key);
 
 /* file.c */
 struct si {
@@ -134,7 +134,7 @@ struct si {
     struct si * last_known;
 };
 void put_saved_items_into_tree (struct si *);
-int reiserfsck_file_write (struct item_head * ih, char * item, int);
+int reiserfsck_file_write (struct item_head * ih, char * item, int was_in_tree);
 int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, __u32 * blocks, int mark_passed_items, int * symlink);
 int delete_N_items_after_key(struct key * start_key, struct si ** save_here, int skip_dir_items, int n_to_delete);
 void rewrite_file (struct item_head * ih, int should_relocate, int should_change_ih);
@@ -200,7 +200,9 @@ void do_clean_attributes (reiserfs_filsys_t * fs);
 int bad_pair (reiserfs_filsys_t *, struct buffer_head * bh, int i);
 int bad_leaf_2 (reiserfs_filsys_t *, struct buffer_head * bh);
 
-
+extern int should_be_relocated (struct key * key);
+extern void to_be_relocated (struct key * key);
+extern void clear_relocated_list(void);
 
 /* ustree.c */
 void reiserfsck_paste_into_item (struct path * path, const char * body, int size);
@@ -217,7 +219,7 @@ void reiserfsck_cut_from_item (struct path * path, int cut_size);
 
 typedef int do_after_read_t (reiserfs_filsys_t *, struct buffer_head **, int h);
 typedef int do_on_full_path_t (reiserfs_filsys_t *, struct buffer_head **, int);
-void pass_through_tree (reiserfs_filsys_t *, do_after_read_t, do_on_full_path_t);
+void pass_through_tree (reiserfs_filsys_t *, do_after_read_t, do_on_full_path_t, int depth);
 
 //int comp_keys_3 (void * key1, void * key2);
 //int comp_dir_entries (void * key1, void * key2);
@@ -241,7 +243,7 @@ int comp_ids(const void *p1, const void *p2);
 int is_objectid_used (reiserfs_filsys_t *, __u32 objectid);
 __u32 get_unused_objectid (reiserfs_filsys_t *);
 
-struct id_map * init_id_map (void);
+struct id_map * init_id_map (__u8);
 void free_id_map (struct id_map *);
 int is_objectid_really_used (struct id_map *, __u32 id, __u32 * ppos);
 int mark_objectid_really_used (struct id_map *, __u32 id);
@@ -252,38 +254,25 @@ void fetch_objectid_map (struct id_map * map, reiserfs_filsys_t * fs);
 void reiserfs_objectid_map_save (FILE * fp, struct id_map * id_map);
 struct id_map * reiserfs_objectid_map_load (FILE * fp);
 
-/* segments.c */
-/*
-struct overwritten_unfm_segment {
-    int ous_begin;
-    int ous_end;
-    struct overwritten_unfm_segment * ous_next;  
-};
-struct overwritten_unfm * look_for_overwritten_unfm (__u32);
-struct overwritten_unfm_segment * find_overwritten_unfm (unsigned long unfm, int length, struct overwritten_unfm_segment * segment_to_init);
-int get_unoverwritten_segment (struct overwritten_unfm_segment * list_head, struct overwritten_unfm_segment * unoverwritten_segment);
-void save_unfm_overwriting (unsigned long unfm, struct item_head * direct_ih);
-void free_overwritten_unfms (void);
-*/
-
 void mark_formatted_pointed_by_indirect (__u32);
 int is_formatted_pointed_by_indirect (__u32);
 
 
 /* size of 1 piece of in-memory map */
-#define MAP_SIZE   4096
+#define MAP_SIZE	4 * 1024
+
+#define MAP_NOT_PACKED	0
+#define MAP_PACKED	1
 
 struct id_map {
-    __u32 * m_begin; /* pointer to map area */
-    __u32 m_used_slots_count;
-    int m_page_count; /* objectid map expands by one page at
-                         time. This is size of objectid map size in
-                         pages */
-    unsigned long objectids_marked; /* number of objectids marked used
-                                       in a map */
+    __u32 * m_begin;		    /* pointer to map area */
+    __u32 m_used_slot_count;	    /* number of used slots in the allocated area */
+    int m_page_count;		    /* number of MAP_SIZE allocated pages */
+    unsigned long objectids_marked; /* number of used objectids */
+    __u8 packed;		    /* pack all usedids into one interval instead 
+				       of map growing */
 };			
 
-
 struct pass0_stat {
     unsigned long dealt_with; /* number of blocks read during pass 0 */
     unsigned long leaves; /* blocks looking like reiserfs leaves found */
@@ -508,8 +497,16 @@ reiserfs_warning (fsck_progress_file(fs), fmt, ## list);\
 fflush (fsck_progress_file(fs));\
 }
 
+#define FATAL	1
+#define FIXABLE 2
+
+void one_more_corruption(reiserfs_filsys_t *fs, int kind);
+void one_less_corruption(reiserfs_filsys_t *fs, int kind);
+
+/*
 #define one_more_corruption(fs,kind) fsck_check_stat (fs)->kind##_corruptions++
 #define one_less_corruption(fs,kind) fsck_check_stat (fs)->kind##_corruptions--
+*/
 
 #define fsck_exit(fmt, list...) \
 {\
