@@ -165,6 +165,7 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
     int version = 0;
     struct reiserfs_super_block * ondisk_sb = 0;
     struct reiserfs_super_block * sb = 0;
+    struct reiserfs_journal_header * jh;
 
     int magic_was_found = 0;
     unsigned long block_count = 0;
@@ -197,7 +198,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 		if ((*tmp && strcmp(tmp, "\n")) || retval < 0)
 		    reiserfs_exit (16, "rebuild_sb: wrong block size specified\n");
 		if (!is_blocksize_correct (retval))
-		    reiserfs_exit (16, "rebuild_sb: wrong block size specified, only divisible by 1024 are supported currently\n");
+		    reiserfs_exit (16, "rebuild_sb: wrong block size specified, only power "
+			    "of 2 from 512-8192 interval are supported.\n");
 	    } else
 		retval = 4096;
 	
@@ -215,7 +217,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
             /* 3_6 magic */
             if (fsck_data (fs)->journal_dev_name)
                 /* journal dev must not be specified with standard journal */
-                reiserfs_exit (16, "Reiserfs with standard journal found, but there was specified a journal dev");
+                reiserfs_exit (16, "Reiserfs with standard journal found, but there was "
+			"specified a journal dev");
 
             if (get_jp_journal_1st_block(sb_jp(sb)) == get_journal_new_start_must (fs))
                 version = 1;
@@ -225,7 +228,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
         } else if (is_reiserfs_3_5_magic_string (sb)) {
             if (fsck_data (fs)->journal_dev_name)
                 /* journal dev must not be specified with standard journal */
-                reiserfs_exit (16, "Reiserfs with standard journal found, but there was specified a journal dev");
+                reiserfs_exit (16, "Reiserfs with standard journal found, but there was "
+			"specified a journal dev");
 
             /* 3_5 magic */
             if (get_jp_journal_1st_block(sb_jp(sb)) == get_journal_new_start_must (fs))
@@ -236,7 +240,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
         } else if (is_reiserfs_jr_magic_string (sb)) {
             if (!fsck_data (fs)->journal_dev_name)
                 /* journal dev must be specified with non standard journal */
-                reiserfs_exit (16, "Reiserfs with non standard journal found, but there was not specified any journal dev");
+                reiserfs_exit (16, "Reiserfs with non standard journal found, but there was "
+		    "not specified any journal dev");
 
             if (get_sb_version (sb) == REISERFS_FORMAT_3_6) {
                 /*non-standard magic + sb_format == 3_6*/
@@ -258,7 +263,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
                 magic_was_found = 3;
             }
         } else
-            reiserfs_exit (16, "we opened device but there is no magic and there is no correct superbblock format found");
+            reiserfs_exit (16, "we opened device but there is no magic and there is no correct "
+		    "superbblock format found");
 
         if (magic_was_found == 1 || magic_was_found == 2)
             standard_journal = 1;
@@ -280,14 +286,15 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
                     block_count = atoi (answer);
                 set_sb_block_count (sb, block_count);
             } else {
-                fsck_conditional_log (magic_was_found, "rebuild-sb: wrong block count occured (%lu), fixed (%lu)\n",
-                     get_sb_block_count(sb), block_count);
+                fsck_conditional_log (magic_was_found, "rebuild-sb: wrong block count occured (%lu), "
+		    "fixed (%lu)\n", get_sb_block_count(sb), block_count);
                 set_sb_block_count (sb, block_count);
             }
         }
 
         if (get_sb_block_size (sb) != fs->fs_blocksize) {
-            fsck_log("rebuild-sb: wrong block size occured (%lu), fixed (%lu)\n", get_sb_block_size (sb), fs->fs_blocksize);
+            fsck_log("rebuild-sb: wrong block size occured (%lu), fixed (%lu)\n", 
+		    get_sb_block_size (sb), fs->fs_blocksize);
   	    set_sb_block_size (sb, fs->fs_blocksize);
         }
     }
@@ -313,7 +320,8 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 		if ((*tmp && strcmp(tmp, "\n")) || retval < 0)
 		    reiserfs_exit (16, "rebuild_sb: wrong block size specified\n");
 		if (!is_blocksize_correct (retval))
-		    reiserfs_exit (16, "rebuild_sb: wrong block size specified, only divisible by 1024 are supported currently\n");
+		    reiserfs_exit (16, "rebuild_sb: wrong block size specified, only divisible "
+			    "by 1024 are supported currently\n");
 	    } else
 		retval = 4096;
 	}
@@ -335,13 +343,17 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
             break;
         }
 
+	if (fs == NULL)
+	    return;
+	
         sb = fs->fs_ondisk_sb;
         fs->fs_vp = data;
 
         if (!fsck_skip_journal (fs)) {
             if (!fsck_data (fs)->journal_dev_name) {
                 do {
-                    printf("\nNo journal device was specified. (If journal is not available, re-run with --no-journal-available).\n"\
+                    printf("\nNo journal device was specified. (If journal is not available, "
+			    "re-run with --no-journal-available).\n"\
                             "Is journal standard? (y/n)[y]: ");
                     getline (&answer, &n, stdin);
                 } while (strcmp ("y\n", answer) && strcmp ("n\n", answer) && strcmp ("\n", answer));
@@ -369,48 +381,57 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
             set_sb_block_count (sb, block_count);
         }
 	
-	set_sb_fs_state (sb, REISERFS_CORRUPTED);
+	set_sb_fs_state (sb, FS_ERROR);
     }
 
 
     if (version == 1 || version == 3) {
         if (get_reiserfs_format (sb) != REISERFS_FORMAT_3_6) {
-            fsck_log("rebuild-sb: wrong reiserfs version occured (%lu), fixed (%lu)\n", get_reiserfs_format (sb), REISERFS_FORMAT_3_6);
+            fsck_log("rebuild-sb: wrong reiserfs version occured (%lu), fixed (%lu)\n", 
+		    get_reiserfs_format (sb), REISERFS_FORMAT_3_6);
             set_sb_version (sb, REISERFS_FORMAT_3_6);
         }
     } else if (version == 2 || version == 4) {
         if (get_reiserfs_format (sb) != REISERFS_FORMAT_3_5) {
-            fsck_log("rebuild-sb: wrong reiserfs version occured (%lu), fixed (%lu)\n", get_reiserfs_format (sb), REISERFS_FORMAT_3_5);
+            fsck_log("rebuild-sb: wrong reiserfs version occured (%lu), fixed (%lu)\n", 
+		    get_reiserfs_format (sb), REISERFS_FORMAT_3_5);
             set_sb_version (sb, REISERFS_FORMAT_3_5);
         }
     }
 
     p_oid_maxsize = (fs->fs_blocksize - reiserfs_super_block_size (sb)) / sizeof(__u32) / 2 * 2;
     if (get_sb_oid_maxsize (sb) != p_oid_maxsize) {
-	fsck_log("rebuild-sb: wrong objectid map max size occured (%lu), fixed (%lu)\n", get_sb_oid_maxsize (sb), p_oid_maxsize);
+	fsck_log("rebuild-sb: wrong objectid map max size occured (%lu), fixed (%lu)\n", 
+		get_sb_oid_maxsize (sb), p_oid_maxsize);
 	set_sb_oid_maxsize (sb, p_oid_maxsize);
     }
 
     p_bmap_nr = (block_count + (fs->fs_blocksize * 8 - 1)) / (fs->fs_blocksize * 8);
     if (get_sb_bmap_nr (sb) != p_bmap_nr) {
-	fsck_log("rebuild-sb: wrong bitmap number occured (%lu), fixed (%lu)\n", get_sb_bmap_nr (sb), p_bmap_nr);
-	set_sb_bmap_nr (sb, (block_count + (fs->fs_blocksize * 8 - 1)) / (fs->fs_blocksize * 8));
+	fsck_log("rebuild-sb: wrong bitmap number occured (%lu), fixed (%lu)\n", 
+		get_sb_bmap_nr (sb), p_bmap_nr);
+	set_sb_bmap_nr (sb, 
+		(block_count + (fs->fs_blocksize * 8 - 1)) / (fs->fs_blocksize * 8));
     }
 
     if (get_sb_root_block (sb) > block_count) {
-	fsck_log("rebuild-sb: wrong root block occured (%lu), zeroed\n", get_sb_root_block (sb));
+	fsck_log("rebuild-sb: wrong root block occured (%lu), zeroed\n", 
+		get_sb_root_block (sb));
 	set_sb_root_block (sb, 0);
     }
 
     if (get_sb_free_blocks (sb) > block_count) {
-	fsck_log ("rebuild-sb: wrong free block count occured (%lu), zeroed\n", get_sb_free_blocks (sb));
+	fsck_log ("rebuild-sb: wrong free block count occured (%lu), zeroed\n", 
+		get_sb_free_blocks (sb));
         set_sb_free_blocks (sb, 0);
     }
 
-    if (get_sb_umount_state (sb) != REISERFS_CLEANLY_UMOUNTED && get_sb_umount_state (sb) != REISERFS_NOT_CLEANLY_UMOUNTED) {
-	fsck_conditional_log (magic_was_found, "rebuild-sb: wrong umount state, fixed to (REISERFS_NOT_CLEANLY_UMOUNTED)\n",
-                  get_sb_umount_state (sb));
-        set_sb_umount_state (sb, REISERFS_NOT_CLEANLY_UMOUNTED);
+    if (get_sb_umount_state (sb) != FS_CLEANLY_UMOUNTED && 
+	get_sb_umount_state (sb) != FS_NOT_CLEANLY_UMOUNTED) 
+    {
+	fsck_conditional_log (magic_was_found, "rebuild-sb: wrong umount state, "
+	    "fixed to (FS_NOT_CLEANLY_UMOUNTED)\n", get_sb_umount_state (sb));
+        set_sb_umount_state (sb, FS_NOT_CLEANLY_UMOUNTED);
     }
 
     if (get_sb_oid_cursize (sb) == 1 || get_sb_oid_cursize (sb) > get_sb_oid_maxsize (sb)) {
@@ -465,68 +486,129 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 
     p_jp_journal_1st_block = get_journal_start_must (fs);
 
-    if (!fsck_skip_journal (fs) && standard_journal == 1) {
+    if (standard_journal == 1) {
         if (get_jp_journal_dev (sb_jp(sb)) != 0) {
- 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal device occured (%lu), fixed (0)\n",
- 	                get_jp_journal_dev (sb_jp(sb)));
+ 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal device occured "
+		"(%lu), fixed (0)\n", get_jp_journal_dev (sb_jp(sb)));
  	    set_jp_journal_dev (sb_jp(sb), 0);
         }
         if (get_sb_reserved_for_journal (sb) != 0) {
- 	    fsck_log ("rebuild-sb: wrong size reserved for standard journal occured (%lu), fixed (0)\n",
- 	                get_sb_reserved_for_journal (sb));
+ 	    fsck_log ("rebuild-sb: wrong size reserved for standard journal occured (%lu), "
+		"fixed (0)\n", get_sb_reserved_for_journal (sb));
             set_sb_reserved_for_journal (sb, 0);
         }
         if (get_jp_journal_1st_block (sb_jp(sb)) != p_jp_journal_1st_block) {
-            fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal first block occured (%lu), fixed (%lu)\n",
-                get_jp_journal_1st_block (sb_jp(sb)), p_jp_journal_1st_block);
+            fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal first block "
+		"occured (%lu), fixed (%lu)\n", get_jp_journal_1st_block (sb_jp(sb)), 
+		p_jp_journal_1st_block);
+	    
             set_jp_journal_1st_block (sb_jp(sb) , p_jp_journal_1st_block);
         }
-        if (get_jp_journal_size (sb_jp(sb)) != journal_default_size(fs->fs_super_bh->b_blocknr, fs->fs_blocksize)) {
- 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal size occured (%lu), fixed (%lu)\n",
- 	                get_jp_journal_size (sb_jp(sb)) + 1,
- 	                journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize) + 1);
-            set_jp_journal_size (sb_jp(sb), journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize));
+        if (get_jp_journal_size (sb_jp(sb)) != 
+	    journal_default_size(fs->fs_super_bh->b_blocknr, fs->fs_blocksize)) 
+	{
+ 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal size occured "
+		"(%lu), fixed (%lu)\n", get_jp_journal_size (sb_jp(sb)) + 1,
+ 	        journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize) + 1);
+	    
+            set_jp_journal_size (sb_jp(sb), 
+		journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize));
         }
+	
         if (get_jp_journal_max_trans_len (sb_jp(sb)) !=
-	    advise_journal_max_trans_len(	get_jp_journal_max_trans_len (sb_jp(sb)),
+	    advise_journal_max_trans_len (	get_jp_journal_max_trans_len (sb_jp(sb)),
 	    					get_jp_journal_size (sb_jp(sb)),
 	    					fs->fs_blocksize))
 	{
- 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal max transaction length occured (%lu), fixed (%d)\n",
- 	        get_jp_journal_max_trans_len (sb_jp(sb)),
-        	advise_journal_max_trans_len (	get_jp_journal_max_trans_len (sb_jp(sb)),
+ 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal max transaction "
+		"length occured (%lu), fixed (%d)\n",
+		get_jp_journal_max_trans_len (sb_jp(sb)),
+		advise_journal_max_trans_len (	get_jp_journal_max_trans_len (sb_jp(sb)),
         					get_jp_journal_size (sb_jp(sb)),
         					fs->fs_blocksize));
+	    
  	    set_jp_journal_max_trans_len (sb_jp(sb),
  	    	advise_journal_max_trans_len(	get_jp_journal_max_trans_len (sb_jp(sb)),
  	    					get_jp_journal_size (sb_jp(sb)),
  	    					fs->fs_blocksize));
         }
+	
 /*        if (get_jp_journal_magic (sb_jp(sb)) != 0) {
- 	    fsck_log ("rebuild-sb: wrong journal magic occured (%lu), fixed (0)\n", get_jp_journal_magic (sb_jp(sb)));
+ 	    fsck_log ("rebuild-sb: wrong journal magic occured (%lu), fixed (0)\n", 
+		get_jp_journal_magic (sb_jp(sb)));
+		
  	    set_jp_journal_magic (sb_jp(sb), 0);
-        }*/
-        if (get_jp_journal_max_batch (sb_jp(sb)) != advise_journal_max_batch(get_jp_journal_max_trans_len (sb_jp(sb)))) {
- 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal max batch size occured (%lu), fixed (%d)\n",
- 	        get_jp_journal_max_batch (sb_jp(sb)), advise_journal_max_batch(get_jp_journal_max_trans_len (sb_jp(sb))));
- 	    set_jp_journal_max_batch (sb_jp(sb), advise_journal_max_batch(get_jp_journal_max_trans_len (sb_jp(sb))));
         }
-        if (get_jp_journal_max_commit_age (sb_jp(sb)) != advise_journal_max_commit_age()) {
- 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal  max commit age occured (%lu), fixed (%d)\n",
+*/
+        if (get_jp_journal_max_batch (sb_jp(sb)) != 
+	    advise_journal_max_batch(get_jp_journal_max_trans_len (sb_jp(sb)))) 
+	{
+ 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal max batch size "
+		"occured (%lu), fixed (%d)\n", get_jp_journal_max_batch (sb_jp(sb)), 
+		advise_journal_max_batch(get_jp_journal_max_trans_len (sb_jp(sb))));
+	    
+ 	    set_jp_journal_max_batch (sb_jp(sb), 
+		advise_journal_max_batch(get_jp_journal_max_trans_len (sb_jp(sb))));
+        }
+        if (get_jp_journal_max_commit_age (sb_jp(sb)) != 
+	    advise_journal_max_commit_age()) 
+	{
+ 	    fsck_conditional_log (magic_was_found, "rebuild-sb: wrong journal "
+		"max commit age occured (%lu), fixed (%d)\n", 
  	        get_jp_journal_max_commit_age (sb_jp(sb)), advise_journal_max_commit_age());
+	    
  	    set_jp_journal_max_commit_age (sb_jp(sb), advise_journal_max_commit_age());
         }
+	
         if (get_jp_journal_max_trans_age (sb_jp(sb)) != advise_journal_max_trans_age()) {
  	    fsck_log ("rebuild-sb: wrong journal  max commit age occured (%lu), fixed (0)\n",
  	        get_jp_journal_max_trans_age (sb_jp(sb)), advise_journal_max_trans_age());
  	    set_jp_journal_max_trans_age (sb_jp(sb), advise_journal_max_trans_age());
         }
 
+	if (reiserfs_open_journal (fs, filename, O_RDONLY) == 0) {
+            fsck_log("\nrebuild-sb: You either have a corrupted journal or have just changed "
+		"the start \nof the partition with some partition table editor. If you are sure "
+		"that the \nstart of the partition is ok, rebuild the journal header.\n");
 
+	    if (!user_confirmed (stdout, "Do you want to rebuild the journal header? "
+		"(y/n)[n]: ", "y\n")) 
+	    {
+		exit(16);
+	    }
+	    
+	    
+	    {
+		struct buffer_head *bh;
 
-    } else if (!fsck_skip_journal(fs) && standard_journal == 0) {
+		bh = bread(fs->fs_dev, get_jp_journal_1st_block (sb_jp (sb)) +
+		    get_jp_journal_size (sb_jp (sb)), fs->fs_blocksize);
+
+		if (bh == NULL) {
+		    die ("Read of the block (%u) failed", 
+			get_jp_journal_1st_block (sb_jp (sb)) + get_jp_journal_size (sb_jp (sb)));
+		}
+		
+		jh = (struct reiserfs_journal_header *)bh->b_data;
+		/* copy 1st block, size and offset to jouranal. */
+		memcpy (&jh->jh_journal, sb_jp(sb), 3 * sizeof(__u32));
+		
+		mark_buffer_uptodate (bh, 1);
+		mark_buffer_dirty (bh);
+		bwrite (bh);		
+		brelse(bh);
+	    }
+	    if (reiserfs_open_journal (fs, filename, O_RDONLY) == 0) 
+		reiserfs_exit(16, "rebuild_sb: failed to open a rebuild sb.");
+	}
+
+	jh = (struct reiserfs_journal_header *)fs->fs_jh_bh->b_data;
+
+	/* copy journal params without 1st block, size and offset. */
+	memcpy (sb_jp(sb) + 3 * sizeof(__u32),  &jh->jh_journal + 3 * sizeof(__u32), 
+	    sizeof (struct journal_params) - 3 * sizeof(__u32));
+    } else if (!fsck_skip_journal(fs)) {
         /* non standard journal */
-        struct reiserfs_journal_header * jh;
 
         journal_dev_name = fsck_data (fs)->journal_dev_name;
         retval = stat(journal_dev_name, &stat_buf);
@@ -559,18 +641,23 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
         }
 */
         retval = -1;
-        if (magic_was_found == 0 || (retval = reiserfs_open_journal (fs, journal_dev_name, O_RDONLY)) == 0) {
+        if (magic_was_found == 0 || 
+	    (retval = reiserfs_open_journal (fs, journal_dev_name, O_RDONLY)) == 0) 
+	{
             __u64 default_value;
-            /* journal header was not found or journal cannot be opened -> adjust journal size and offset */
+            /* journal header was not found or journal cannot be opened -> adjust journal 
+	     * size and offset */
 
-            /* default offset if magic was not found is 0 for relocated journal and get_journal_start_must
-               for any journal on the same device;
+            /* default offset if magic was not found is 0 for relocated journal and 
+	     * get_journal_start_must for any journal on the same device;
                default offset if magic was found is found value */
             if (retval == 0)
-                fsck_log ("Journal cannot be opened, assuming specified journal device is correct\n");
+                fsck_log ("Journal cannot be opened, assuming specified journal device "
+		    "is correct\n");
 
             if (magic_was_found == 0)
-                default_value = (!strcmp(fs->fs_file_name, journal_dev_name)) ? p_jp_journal_1st_block : 0;
+                default_value = (!strcmp(fs->fs_file_name, journal_dev_name)) ? 
+				p_jp_journal_1st_block : 0;
             else
                 default_value = get_jp_journal_1st_block (sb_jp(sb));
 
@@ -597,14 +684,14 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
                default size if magic was found is found value + 1 block for journal header */
             if (magic_was_found == 0)
                 default_value = (!strcmp(fs->fs_file_name, journal_dev_name)) ?
-                                journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize) + 1 :
+                                journal_default_size(fs->fs_super_bh->b_blocknr,fs->fs_blocksize)+1:
                                 p_jp_dev_size - get_jp_journal_1st_block (sb_jp(sb));
             else
                 default_value = get_jp_journal_size (sb_jp(sb)) + 1;
 		
 
-            printf("\nEnter journal size (including 1 block for journal header) on %s in blocks [%Lu]: \n",
-                        journal_dev_name, (unsigned long long)default_value);
+            printf("\nEnter journal size (including 1 block for journal header) on %s in blocks "
+		"[%Lu]: \n", journal_dev_name, (unsigned long long)default_value);
 
             getline (&answer, &n, stdin);
             if (strcmp(answer, "\n")) {
@@ -619,16 +706,62 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
             /* some checks for journal size */
             if (get_jp_journal_size (sb_jp(sb)) +
                 get_jp_journal_1st_block (sb_jp(sb)) + 1 > p_jp_dev_size)
-        	    reiserfs_exit(16, "rebuild_sb: journal offset + journal size is much then device size\n");
+        	    reiserfs_exit(16, "rebuild_sb: journal offset + journal size is much then "
+			"device size\n");
 
-            if (reiserfs_open_journal (fs, journal_dev_name, O_RDONLY) == 0)
-                reiserfs_exit(16, "rebuild-sb: journal header is not found, wrong dev/offset/size configuration\n");
+            if (reiserfs_open_journal (fs, journal_dev_name, O_RDONLY) == 0) {
+		fsck_log("\nrebuild-sb: journal header is not found. You either have a corrupted,\n"
+		    "bad configured(device/offset/size), not available journal or have just changed\n"
+		    "the start of the journal partition with some partition table editor. In the \n"
+		    "case of corrupted journal you need to use --no-journal-available. If you are\n"
+		    "sure that the start of the partition is ok and journal is available, rebuild\n"
+		    "the journal header.\n");
+
+		if (!user_confirmed (stdout, "Do you want to rebuild the journal header? "
+			"(y/n)[n]: ", "y\n")) 
+		{
+		    exit(16);
+		}
+
+		{
+		    struct buffer_head *bh;
+		    int fs_journal_dev;
+
+		    fs_journal_dev = open (journal_dev_name, O_RDONLY | O_LARGEFILE);
+		    
+		    bh = bread(fs_journal_dev, get_jp_journal_1st_block (sb_jp (sb)) +
+			get_jp_journal_size (sb_jp (sb)), fs->fs_blocksize);
+
+		    if (bh == NULL) {
+			die ("Read of the block (%u) failed", 
+			    get_jp_journal_1st_block (sb_jp (sb)) + get_jp_journal_size (sb_jp (sb)));
+		    }
+		    
+		    jh = (struct reiserfs_journal_header *)bh->b_data;
+		    /* copy 1st block, size and offset to jouranal. */
+		    memcpy (&jh->jh_journal, sb_jp(sb), 3 * sizeof(__u32));
+		
+		    mark_buffer_uptodate (bh, 1);
+		    mark_buffer_dirty (bh);
+		    bwrite (bh);		
+		    brelse(bh);
+		    close(fs_journal_dev);
+		}
+	    
+		if (reiserfs_open_journal (fs, filename, O_RDONLY) == 0) 
+		    reiserfs_exit(16, "rebuild_sb: failed to open a rebuild sb.");
+	    }
         }
 
 	jh = (struct reiserfs_journal_header *)fs->fs_jh_bh->b_data;
-        memcpy (sb_jp(sb), &jh->jh_journal, sizeof (struct journal_params));
+
+	/* copy journal params without 1st block, size and offset from journal to sb. */
+	memcpy (sb_jp(sb) + 3 * sizeof(__u32), &jh->jh_journal + 3 * sizeof(__u32), 
+	    sizeof (struct journal_params) - 3 * sizeof(__u32));
     } else {
-        fsck_log ("Journal was specified as not available, zeroing all journal fields in super block..");
+        fsck_log ("\nJournal was specified as not available, zeroing all journal fields "
+	    "in super block.\n\n");
+	
         set_jp_journal_dev (sb_jp(sb), 0);
         set_sb_reserved_for_journal (sb, 0);
         set_jp_journal_1st_block (sb_jp(sb) , 0);
@@ -651,7 +784,7 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 		((sb_size == SB_SIZE) ? sizeof(fs->fs_ondisk_sb->s_unused) : 0)) ) 
     {
         /* smth was changed in SB or a new one has been built */
-	set_sb_fs_state (sb, get_sb_fs_state (sb) || REISERFS_CORRUPTED);
+	set_sb_fs_state (sb, get_sb_fs_state (sb) | FS_ERROR);
 	
 	if (ondisk_sb) {
 	    /* if super_block was found, we keep sb in ondisk_sb */
@@ -660,13 +793,14 @@ void rebuild_sb (reiserfs_filsys_t * fs, char * filename, struct fsck_data * dat
 	    freemem(sb);
 	}
 
+	fflush(stdout); 
 	print_block (stderr, fs, fs->fs_super_bh);
 	
 	if (user_confirmed (stderr, "Is this ok ? (y/n)[n]: ", "y\n")) {
 	    mark_buffer_uptodate (fs->fs_super_bh, 1);
 	    mark_buffer_dirty (fs->fs_super_bh);
 	    bwrite (fs->fs_super_bh);
-	    fsck_progress ("\nDo not forget to run reiserfsck --rebuild-tree\n\n");
+	    fsck_progress ("The fs may still be unconsistent. Run reiserfsck --check.\n\n");
 	} else {
 	    mark_buffer_clean (fs->fs_super_bh);
 	    fsck_progress ("Super block was not written\n");
