@@ -1,5 +1,5 @@
 /*
- * Copyright 1996-2002 Hans Reiser
+ * Copyright 1996-2003 Hans Reiser
  */
 
 /* nothing abount reiserfs here */
@@ -49,27 +49,32 @@ int generate_random_uuid (unsigned char * uuid);
 int uuid_is_correct (unsigned char * uuid);
 int set_uuid (const unsigned char * text, unsigned char * UUID);
 
-#include <asm/bitops.h>
+//#include <asm/bitops.h>
+
+extern inline int misc_set_bit (unsigned long long nr, void * addr);
+extern inline int misc_clear_bit (unsigned long long nr, void * addr);
+extern inline int misc_test_bit(unsigned long long nr, const void * addr);
+extern inline unsigned long long misc_find_first_zero_bit (const void *vaddr, unsigned long long size);
+extern inline unsigned long long misc_find_next_zero_bit (const void *vaddr, unsigned long long size, unsigned long long offset);
+    
 #if __BYTE_ORDER == __LITTLE_ENDIAN
-int le_set_bit (int nr, void * addr);
-int le_clear_bit (int nr, void * addr);
-int le_test_bit(int nr, const void * addr);
-int le_find_first_zero_bit (const void *vaddr, unsigned size);
-int le_find_next_zero_bit (const void *vaddr, unsigned size, unsigned offset);
+
 # define cpu_to_le16(val)                 (val)
 # define le16_to_cpu(val)                 (val)
 # define cpu_to_le32(val)                 (val)
 # define le32_to_cpu(val)                 (val)
 # define cpu_to_le64(val)                 (val)
 # define le64_to_cpu(val)                 (val)
+
 #elif __BYTE_ORDER == __BIG_ENDIAN
 
-# define cpu_to_le16(val)                 swab16(val)
-# define le16_to_cpu(val)                 swab16(val)
-# define cpu_to_le32(val)                 swab32(val)
-# define le32_to_cpu(val)                 swab32(val)
-# define cpu_to_le64(val)                 swab64(val)
-# define le64_to_cpu(val)                 swab64(val)
+# define cpu_to_le16(val)                 __swab16(val)
+# define le16_to_cpu(val)                 __swab16(val)
+# define cpu_to_le32(val)                 __swab32(val)
+# define le32_to_cpu(val)                 __swab32(val)
+# define cpu_to_le64(val)                 __swab64(val)
+# define le64_to_cpu(val)                 __swab64(val)
+
 #else
 # error "nuxi/pdp-endian archs are not supported"
 #endif
