@@ -1,5 +1,5 @@
 /*
- * Copyright 2002 Hans Reiser
+ * Copyright 2002-2003 Hans Reiser
  */
 
 #include "includes.h"
@@ -486,8 +486,10 @@ int reiserfs_open_journal (reiserfs_filsys_t * fs, char * j_filename, int flags)
         get_jp_journal_dev (&j_head->jh_journal) != get_jp_journal_dev (sb_jp (sb)) || 
         get_jp_journal_size (&j_head->jh_journal) != get_jp_journal_size (sb_jp (sb))) 
     {
-	reiserfs_warning (stderr, "reiserfs_open_journal: journal parameters from super block does not match to\n"
-	    "journal parameters from journal.\n", j_filename);
+	reiserfs_warning (stderr, 
+	    "\nreiserfs_open_journal: journal parameters from the super block does not match \n"
+	    "to journal parameters from the journal. It is expected if you created your fs \n"
+	    "with old reiserfsprogs. In this case rebuild-sb will fix it all for you.\n", j_filename);
         brelse (bh);
         return 0;
         
