@@ -1,5 +1,5 @@
 /*
- * Copyright 2002  Hans Reiser
+ * Copyright 2002-2003 Hans Reiser
  */
 #include "tune.h"
 
@@ -99,9 +99,11 @@ static int should_make_journal_standard (reiserfs_filsys_t * fs, char * j_new_de
 		    get_jp_journal_dev(sb_jp(fs->fs_ondisk_sb)));
 	    return 0;
 	}
-	if(get_jp_journal_size(sb_jp(fs->fs_ondisk_sb))!= journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize)){
-	    message ("Can not create standard journal of the size %u",
-		     get_jp_journal_size(sb_jp(fs->fs_ondisk_sb)));
+	if (get_sb_reserved_for_journal(fs->fs_ondisk_sb) < 
+	    journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize) + 1)
+	{
+	    message ("Can not create standard journal of the size %lu",
+		     journal_default_size (fs->fs_super_bh->b_blocknr, fs->fs_blocksize) + 1);
 	    return 0;
 	}
     }
@@ -110,6 +112,8 @@ static int should_make_journal_standard (reiserfs_filsys_t * fs, char * j_new_de
 
 static int set_standard_journal_params (reiserfs_filsys_t * fs)
 {
+    struct buffer_head * bh;
+
     /* ondisk superblock update */
 
     if (get_sb_version(fs->fs_ondisk_sb) == 0)
@@ -123,6 +127,11 @@ static int set_standard_journal_params (reiserfs_filsys_t * fs)
 		 " try reiserfsck first", get_sb_version(fs->fs_ondisk_sb));
 	return 0;
     }
+    
+    set_jp_journal_1st_block (sb_jp(fs->fs_ondisk_sb), get_journal_start_must (fs));
+    set_jp_journal_dev (sb_jp(fs->fs_ondisk_sb), 0);
+    set_jp_journal_size (sb_jp(fs->fs_ondisk_sb), journal_default_size(fs->fs_super_bh->b_blocknr, fs->fs_blocksize));
+
     if (get_jp_journal_max_trans_len(sb_jp(fs->fs_ondisk_sb)) != JOURNAL_TRANS_MAX)
 	set_jp_journal_max_trans_len(sb_jp(fs->fs_ondisk_sb), JOURNAL_TRANS_MAX);
     if (get_jp_journal_max_batch(sb_jp(fs->fs_ondisk_sb)) != JOURNAL_MAX_BATCH)
@@ -134,9 +143,20 @@ static int set_standard_journal_params (reiserfs_filsys_t * fs)
     set_sb_reserved_for_journal (fs->fs_ondisk_sb, 0);
     
     /* journal_header update */
+    bh = getblk(fs->fs_journal_dev, 
+	    get_jp_journal_1st_block(sb_jp(fs->fs_ondisk_sb)) + 
+	    get_jp_journal_size(sb_jp(fs->fs_ondisk_sb)), fs->fs_blocksize);
     
-    ((struct reiserfs_journal_header *)(fs->fs_jh_bh->b_data)) -> jh_journal =
-	*(sb_jp(fs->fs_ondisk_sb));
+    if (!bh) {
+	message ("Cannot get the journal header block. getblk failed.\n");
+	return 0;
+    }
+    ((struct reiserfs_journal_header *)(bh->b_data))->jh_journal = *(sb_jp(fs->fs_ondisk_sb));
+    mark_buffer_uptodate (bh, 1);
+    mark_buffer_dirty (bh);
+    bwrite(bh);
+    brelse(bh);
+
     return 1;
 }
 
@@ -434,7 +454,9 @@ int main (int argc, char **argv)
 	print_journal_params (stdout, sb_jp (fs->fs_ondisk_sb));
 
 	if ((Options & OPT_STANDARD)
-	    && should_make_journal_standard(fs, j_new_device_name)) {
+	    && should_make_journal_standard(fs, j_new_device_name)) 
+	{
+	    fs->fs_journal_dev = fs->fs_dev;
 	    if (set_standard_journal_params (fs)) {
 		printf ("\nNew parameters:\n");
 		print_filesystem_state (stdout, fs);
@@ -443,8 +465,6 @@ int main (int argc, char **argv)
 		print_journal_params (stdout, sb_jp (fs->fs_ondisk_sb));
 		mark_buffer_dirty (fs->fs_super_bh);
 		mark_buffer_uptodate (fs->fs_super_bh, 1);
-		mark_buffer_dirty (fs->fs_jh_bh);
-		mark_buffer_uptodate (fs->fs_jh_bh, 1);
 		reiserfs_close (fs);
 		printf ("Syncing.."); fflush (stdout);
 		sync ();
